import { Subject } from 'rxjs';
import { coerceBooleanProperty } from '@angular/cdk/coercion';
import { ContentObserver, ObserversModule } from '@angular/cdk/observers';
import { Component, ElementRef, InjectionToken, Input, ViewEncapsulation, Inject, ContentChild, EventEmitter, Output, ViewChild, NgModule } from '@angular/core';
import { CdkOverlayOrigin } from '@angular/cdk/overlay';
import { NtOverlayComponent, NtOverlayPosition, NtOverlayModule } from '@ng-tangram/components/core';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
/** @type {?} */
const NT_DROPDOWN_PARENT_COMPONENT = new InjectionToken('nt-dropdown-parent-component');
class NtDropdownPaneComponent {
    /**
     * @param {?} _contentObserver
     * @param {?} _elementRef
     * @param {?} _parent
     */
    constructor(_contentObserver, _elementRef, _parent) {
        this._contentObserver = _contentObserver;
        this._elementRef = _elementRef;
        this._parent = _parent;
        this._arrow = false;
        this._autosize = false;
        this.size = 'small';
        this.class = '';
        this._contentChange = new Subject();
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set autosize(value) {
        this._autosize = coerceBooleanProperty(value);
    }
    /**
     * @return {?}
     */
    get autosize() { return this._autosize; }
    /**
     * @param {?} value
     * @return {?}
     */
    set arrow(value) { this._arrow = coerceBooleanProperty(value); }
    /**
     * @return {?}
     */
    get arrow() { return this._arrow; }
    /**
     * @return {?}
     */
    get contentChanged() {
        return this._contentChange.asObservable();
    }
    /**
     * @return {?}
     */
    get textContent() {
        return (this._elementRef.nativeElement.textContent || '').trim();
    }
    /**
     * @return {?}
     */
    ngAfterContentInit() {
        this._contentSubscription = this._contentObserver
            .observe(this._elementRef)
            .subscribe(() => this._checkContentChange());
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this._contentSubscription) {
            this._contentSubscription.unsubscribe();
        }
    }
    /**
     * @return {?}
     */
    _checkContentChange() {
        if (this._parent.overlay.isOpen) {
            /** @type {?} */
            const cdkConnectedOverlay = this._parent.overlay.cdkConnectedOverlay;
            if (cdkConnectedOverlay && cdkConnectedOverlay.overlayRef) {
                cdkConnectedOverlay.overlayRef.updatePosition();
            }
        }
    }
}
NtDropdownPaneComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-dropdown-pane, [nt-dropdown-pane]',
                template: '<ng-content></ng-content>',
                encapsulation: ViewEncapsulation.None,
                host: {
                    '[class]': '["nt-dropdown-pane", size, class].join(" ")',
                    '[class.autosize]': 'autosize'
                }
            },] },
];
/** @nocollapse */
NtDropdownPaneComponent.ctorParameters = () => [
    { type: ContentObserver },
    { type: ElementRef },
    { type: undefined, decorators: [{ type: Inject, args: [NT_DROPDOWN_PARENT_COMPONENT,] }] }
];
NtDropdownPaneComponent.propDecorators = {
    size: [{ type: Input }],
    class: [{ type: Input }],
    autosize: [{ type: Input }],
    arrow: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
class NtDropdownComponent {
    /**
     * @param {?} _elementRef
     */
    constructor(_elementRef) {
        this.position = NtOverlayPosition.BottomLeft;
        this.trigger = 'hover';
        this.afterOpen = new EventEmitter();
        this.afterClosed = new EventEmitter();
        this.beforeOpen = new EventEmitter();
        this.beforeClosed = new EventEmitter();
        this.positionChange = new EventEmitter();
        this.origin = new CdkOverlayOrigin(_elementRef);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    _afterOpen(event) {
        this.afterOpen.next(event);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    _afterClosed(event) {
        this.afterClosed.next(event);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    _beforeOpen(event) {
        this.beforeOpen.next(event);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    _beforeClosed(event) {
        this.beforeClosed.next(event);
    }
    /**
     * @param {?} change
     * @return {?}
     */
    _positionChange(change) {
        this.positionChange.next(change);
    }
}
NtDropdownComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-dropdown, [nt-dropdown]',
                template: "<ng-content></ng-content> <nt-overlay [origin]=\"origin\" [position]=\"position\" [trigger]=\"trigger\" [arrow]=\"pane?.arrow\" (afterOpen)=\"_afterOpen($event)\" (afterClosed)=\"_afterClosed($event)\" (beforeOpen)=\"_beforeOpen($event)\" (beforeClosed)=\"_beforeClosed($event)\" (positionChange)=\"_positionChange($event)\"> <ng-content select=\"nt-dropdown-pane\"></ng-content> </nt-overlay> ",
                encapsulation: ViewEncapsulation.None,
                host: {
                    '(click)': 'overlay.click()',
                    '(mouseenter)': 'overlay.onMouseEnter()',
                    '(mouseleave)': 'overlay.onMouseLeave()'
                },
                providers: [
                    { provide: NT_DROPDOWN_PARENT_COMPONENT, useExisting: NtDropdownComponent }
                ]
            },] },
];
/** @nocollapse */
NtDropdownComponent.ctorParameters = () => [
    { type: ElementRef }
];
NtDropdownComponent.propDecorators = {
    position: [{ type: Input }],
    trigger: [{ type: Input }],
    afterOpen: [{ type: Output }],
    afterClosed: [{ type: Output }],
    beforeOpen: [{ type: Output }],
    beforeClosed: [{ type: Output }],
    positionChange: [{ type: Output }],
    overlay: [{ type: ViewChild, args: [NtOverlayComponent,] }],
    pane: [{ type: ContentChild, args: [NtDropdownPaneComponent,] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
class NtDropdownModule {
}
NtDropdownModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule, ObserversModule, NtOverlayModule],
                entryComponents: [NtDropdownComponent],
                exports: [NtDropdownComponent, NtDropdownPaneComponent],
                declarations: [NtDropdownComponent, NtDropdownPaneComponent]
            },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */

export { NtDropdownModule, NtDropdownComponent, NT_DROPDOWN_PARENT_COMPONENT, NtDropdownPaneComponent };
//# sourceMappingURL=dropdown.js.map
